// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

/*
This is an example on how the Azure Go library can be used to interact with
the Windows Azure Service.
Note that this is a provided only as an example and that real code should
probably do something more sensible with errors than ignoring them or panicking.
*/
package main

import (
    "flag"
    "fmt"
    "launchpad.net/gwacl"
    "math/rand"
    "os"
    "time"
)

var certFile string
var subscriptionID string
var verbose bool

func getParams() error {
    flag.StringVar(&certFile, "cert", "", "Name of your management certificate file (in PEM format).")
    flag.StringVar(&subscriptionID, "subscriptionid", "", "Your Azure subscription ID.")
    flag.BoolVar(&verbose, "verbose", false, "Print debugging output")

    flag.Parse()

    if certFile == "" {
        return fmt.Errorf("No .pem certificate specified.  Use the -cert option.")
    }
    if subscriptionID == "" {
        return fmt.Errorf("No subscription ID specified.  Use the -subscriptionid option.")
    }
    return nil
}

func checkError(err error) {
    if err != nil {
        panic(err)
    }
}

// makeRandomIdentifier creates an arbitrary identifier of the given length,
// consisting of only ASCII digits and lower-case ASCII letters.
// The identifier will start with the given prefix.  The prefix must be no
// longer than the specified length, or there'll be trouble.
func makeRandomIdentifier(prefix string, length int) string {
    // Only digits and lower-case ASCII letters are accepted.
    const chars = "abcdefghijklmnopqrstuvwxyz0123456789"

    if len(prefix) > length {
        panic(fmt.Errorf("prefix '%s' is more than the requested %d characters long", prefix, length))
    }

    id := prefix
    for len(id) < length {
        id += string(chars[rand.Intn(len(chars))])
    }
    return id
}

func main() {
    rand.Seed(int64(time.Now().Nanosecond()))

    err := getParams()
    if err != nil {
        fmt.Println(err)
        os.Exit(1)
    }

    api, err := gwacl.NewManagementAPI(subscriptionID, certFile)
    gwacl.SetVerbose(verbose)
    checkError(err)

    // Exercise the API.

    // Exercise hosted services API.
    ExerciseHostedServicesAPI(api)

    fmt.Println("All done.")
}

func ExerciseHostedServicesAPI(api *gwacl.ManagementAPI) {
    location := "East US"
    release := "12.04"

    fmt.Printf("Getting OS Image for release '%s' and location '%s'...\n", release, location)
    images, err := api.ListOSImages()
    checkError(err)
    preciseImage, err := images.GetLatestUbuntuImage(release, location)
    checkError(err)
    sourceImageName := preciseImage.Name
    fmt.Printf("Got image named '%s'\n", sourceImageName)
    fmt.Println("Done getting OS Image\n")

    // A hosted-service name must be between 3 and 24 characters long.
    hostServiceName := makeRandomIdentifier("gwacl", 24)
    fmt.Printf("Creating a hosted service: '%s'...\n", hostServiceName)
    createHostedService := gwacl.NewCreateHostedServiceWithLocation(hostServiceName, "testLabel", location)
    err = api.AddHostedService(createHostedService)
    checkError(err)
    fmt.Println("Done creating a hosted service\n")

    defer func() {
        fmt.Println("Deleting hosted service...")
        api.DeleteHostedService(hostServiceName)
        fmt.Println("Done deleting hosted service\n")
    }()

    fmt.Println("Listing hosted services...")
    hostedServices, err := api.ListHostedServices()
    checkError(err)
    fmt.Printf("Got %d hosted service(s)\n", len(hostedServices))
    if len(hostedServices) > 0 {
        hostedService := hostedServices[0]
        detailedHostedService, err := api.GetHostedServiceProperties(hostedService.ServiceName, true)
        checkError(err)
        fmt.Printf("Hosted service '%s' contains %d deployments\n", hostedService.ServiceName, len(detailedHostedService.Deployments))
        // Do the same again with ListAllDeployments.
        deployments, err := api.ListAllDeployments(&gwacl.ListAllDeploymentsRequest{ServiceName: hostedService.ServiceName})
        checkError(err)
        if len(deployments) != len(detailedHostedService.Deployments) {
            fmt.Printf(
                "Mismatch in reported deployments: %d != %d",
                len(deployments), len(detailedHostedService.Deployments))
        }
    }
    fmt.Println("Done listing hosted services\n")

    fmt.Println("Adding VM deployment...")
    hostname := makeRandomIdentifier("gwaclhost", 25)
    // Azure is strict about passwords, so make it really ugly.
    password := makeRandomIdentifier("Password-!£$%^&*-", 32)
    vhdName := makeRandomIdentifier("gwacldisk", 16)
    configurationSet := gwacl.NewLinuxProvisioningConfiguration(hostname, "test", password, false)

    storageAccount := makeRandomIdentifier("gwacl", 24)
    storageLabel := makeRandomIdentifier("gwacl", 64)
    fmt.Printf("Requesting storage account with name '%s' and label '%s'...\n", storageAccount, storageLabel)
    cssi := gwacl.NewCreateStorageServiceInputWithLocation(storageAccount, storageLabel, location)
    cssi.GeoReplicationEnabled = "false"
    err = api.AddStorageAccount(cssi)
    checkError(err)
    fmt.Println("Done requesting storage account\n")

    defer func() {
        fmt.Printf("Deleting storage account %s...\n", storageAccount)
        api.DeleteStorageAccount(storageAccount)
        fmt.Println("Done deleting storage account\n")
    }()

    mediaLink := fmt.Sprintf("http://%s.blob.core.windows.net/vhds/%s.vhd", storageAccount, vhdName)
    diskName := makeRandomIdentifier("gwacldisk", 16)
    diskLabel := makeRandomIdentifier("gwacl", 64)
    vhd := gwacl.NewOSVirtualHardDisk("", diskLabel, diskName, mediaLink, sourceImageName, "Linux")
    roleName := makeRandomIdentifier("gwaclrole", 16)
    role := gwacl.NewRole("ExtraSmall", roleName, []gwacl.LinuxProvisioningConfiguration{*configurationSet}, []gwacl.OSVirtualHardDisk{*vhd})
    machineName := makeRandomIdentifier("gwaclmachine", 20)
    deployment := gwacl.NewDeploymentForCreateVMDeployment(machineName, "Staging", machineName, []gwacl.Role{*role}, "")
    err = api.AddDeployment(deployment, hostServiceName)
    checkError(err)
    fmt.Println("Done adding VM deployment\n")

    defer func() {
        fmt.Println("Deleting VM disk...")
        err = api.DeleteDisk(diskName)
        checkError(err)
        fmt.Println("Done deleting VM disk\n")
    }()

    defer func() {
        fmt.Println("Deleting deployment...")
        err = api.DeleteDeployment(hostServiceName, deployment.Name)
        checkError(err)
        fmt.Println("Done deleting deployment\n")
    }()

    fmt.Println("Starting VM...")
    err = api.StartRole(&gwacl.StartRoleRequest{hostServiceName, deployment.Name, role.RoleName})
    checkError(err)
    fmt.Println("Done starting VM\n")

    defer func() {
        fmt.Println("Stopping VM...")
        err = api.ShutdownRole(&gwacl.ShutdownRoleRequest{hostServiceName, deployment.Name, role.RoleName})
        checkError(err)
        fmt.Println("Done stopping VM\n")
    }()

    fmt.Println("Listing VM...")
    instances, err := api.ListInstances(&gwacl.ListInstancesRequest{hostServiceName})
    checkError(err)
    fmt.Printf("Got %d instance(s)\n", len(instances))
    fmt.Println("Done listing VM\n")
}
