// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "encoding/xml"
    "errors"
    "fmt"
    . "launchpad.net/gocheck"
    "net/http"
    "strings"
    "time"
)

type managementBaseAPISuite struct {
    x509DispatcherFixture
    oldPollerInterval time.Duration
}

var _ = Suite(&managementBaseAPISuite{})

func makeX509ResponseWithOperationHeader(operationID string) *x509Response {
    header := http.Header{}
    header.Set(operationIDHeaderName, operationID)
    response := x509Response{
        StatusCode: http.StatusAccepted,
        Header:     header,
    }
    return &response
}

func makeAPI(c *C) *ManagementAPI {
    subscriptionId := "subscriptionId"
    api, err := NewManagementAPI(subscriptionId, "certFile")
    c.Assert(err, IsNil)
    // Polling is disabled by default.
    api.PollerInterval = 0
    return api
}

func (suite *managementBaseAPISuite) TestGetOperationIDExtractsHeader(c *C) {
    operationID := "operationID"
    response := makeX509ResponseWithOperationHeader(operationID)

    returnedOperationID, err := getOperationID(response)
    c.Assert(err, IsNil)
    c.Check(returnedOperationID, Equals, operationID)
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedSucceedsOnSyncSuccess(c *C) {
    // Set of expected error returns for success statuses.
    // (They all map to nil.  It makes it easier further down to report
    // unexpected errors in a helpful way).
    expectedErrors := map[int]error{
        http.StatusOK:        nil,
        http.StatusCreated:   nil,
        http.StatusNoContent: nil,
    }
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    receivedErrors := make(map[int]error)
    for status := range expectedErrors {
        err := api.blockUntilCompleted(&x509Response{StatusCode: status})
        receivedErrors[status] = err
    }

    c.Check(receivedErrors, DeepEquals, expectedErrors)
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedReturnsHTTPErrorOnSyncFailure(c *C) {
    // Set of failure statuses, and whether they're supposed to return
    // HTTPError.
    // (They all map to true.  It makes it easier further down to report
    // failures in a helpful way).
    expectedErrors := map[int]bool{
        http.StatusBadRequest:          true,
        http.StatusUnauthorized:        true,
        http.StatusForbidden:           true,
        http.StatusNotFound:            true,
        http.StatusConflict:            true,
        http.StatusInternalServerError: true,
        http.StatusNotImplemented:      true,
    }
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    receivedErrors := make(map[int]bool)
    for status := range expectedErrors {
        err := api.blockUntilCompleted(&x509Response{StatusCode: status})
        _, ok := err.(HTTPError)
        receivedErrors[status] = ok
    }

    c.Check(receivedErrors, DeepEquals, expectedErrors)
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedPollsOnAsyncOperation(c *C) {
    const operationID = "async-operation-id"
    // We set the dispatcher up for failure, and prove that
    // blockUntilCompleted() makes a polling request.
    failure := errors.New("Simulated failure")
    rigFailingDispatcher(failure)
    requests := make([]*X509Request, 0)
    rigRecordingDispatcher(&requests)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond
    accepted := makeX509ResponseWithOperationHeader(operationID)

    err := api.blockUntilCompleted(accepted)

    c.Check(err, Equals, failure)
    c.Assert(len(requests), Equals, 1)
    requestURL := requests[0].URL
    urlParts := strings.Split(requestURL, "/")
    polledOperationID := urlParts[len(urlParts)-1]
    c.Check(polledOperationID, Equals, operationID)
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedErrorsIfAsyncOperationFails(c *C) {
    response := DispatcherResponse{
        response: &x509Response{
            Body:       []byte(fmt.Sprintf(operationXMLTemplate, "Failed")),
            StatusCode: http.StatusOK,
        },
        errorObject: nil}
    responses := []DispatcherResponse{response}
    rigPreparedResponseDispatcher(responses)
    operationID := "operationID"
    operationResponse := makeX509ResponseWithOperationHeader(operationID)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    err := api.blockUntilCompleted(operationResponse)
    c.Check(err, ErrorMatches, ".*asynchronous operation failed.*")
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedErrorsOnInvalidXML(c *C) {
    response := DispatcherResponse{
        response: &x509Response{
            Body:       []byte(">invalidXML<"),
            StatusCode: http.StatusOK,
        },
        errorObject: nil}
    responses := []DispatcherResponse{response}
    rigPreparedResponseDispatcher(responses)
    operationID := "operationID"
    operationResponse := makeX509ResponseWithOperationHeader(operationID)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    err := api.blockUntilCompleted(operationResponse)
    c.Check(err, FitsTypeOf, new(xml.SyntaxError))
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedErrorsIfPollingFails(c *C) {
    response := DispatcherResponse{
        response:    &x509Response{},
        errorObject: fmt.Errorf("unexpected error")}
    responses := []DispatcherResponse{response}
    rigPreparedResponseDispatcher(responses)
    operationID := "operationID"
    operationResponse := makeX509ResponseWithOperationHeader(operationID)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    err := api.blockUntilCompleted(operationResponse)
    c.Check(err, ErrorMatches, ".*unexpected error.*")
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedErrorsCanTimeout(c *C) {
    response := &x509Response{
        Body:       []byte(fmt.Sprintf(operationXMLTemplate, InProgressOperationStatus)),
        StatusCode: http.StatusOK,
    }
    rigFixedResponseDispatcher(response)
    operationID := "operationID"
    operationResponse := makeX509ResponseWithOperationHeader(operationID)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond
    api.PollerTimeout = 2 * time.Nanosecond

    err := api.blockUntilCompleted(operationResponse)
    c.Check(err, ErrorMatches, ".*polling timed out.*")
}

func (suite *managementBaseAPISuite) TestBlockUntilCompletedSucceedsIfAsyncOperationSucceeds(c *C) {
    response := DispatcherResponse{
        response: &x509Response{
            Body:       []byte(fmt.Sprintf(operationXMLTemplate, "Succeeded")),
            StatusCode: http.StatusOK,
        },
        errorObject: nil}
    responses := []DispatcherResponse{response}
    rigPreparedResponseDispatcher(responses)
    operationID := "operationID"
    operationResponse := makeX509ResponseWithOperationHeader(operationID)
    api := makeAPI(c)
    api.PollerInterval = time.Nanosecond

    err := api.blockUntilCompleted(operationResponse)
    c.Assert(err, IsNil)
}

func (suite *managementBaseAPISuite) TestNewManagementAPI(c *C) {
    subscriptionId := "subscriptionId"
    certFile := "certFile"

    api, err := NewManagementAPI(subscriptionId, certFile)

    c.Assert(err, IsNil)
    session, err := newX509Session(subscriptionId, certFile)
    c.Assert(api.session.subscriptionId, DeepEquals, session.subscriptionId)
    c.Assert(api.session.certFile, DeepEquals, session.certFile)
}

func (suite *managementBaseAPISuite) TestNewManagementAPISetsDefaultPollerInterval(c *C) {
    api, err := NewManagementAPI("subscriptionId", "certFile")
    c.Assert(err, IsNil)

    c.Assert(api.PollerInterval, Equals, DefaultPollerInterval)
}

func (suite *managementBaseAPISuite) TestNewManagementAPISetsDefaultPollerTimeout(c *C) {
    api, err := NewManagementAPI("subscriptionId", "certFile")
    c.Assert(err, IsNil)

    c.Assert(api.PollerTimeout, Equals, DefaultPollerTimeout)
}

// setUpDispatcher sets up a request dispatcher that:
// - records requests
// - returns empty responses
func (suite *managementBaseAPISuite) setUpDispatcher() *[]*X509Request {
    fixedResponse := x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte{},
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)
    return &recordedRequests
}

// checkOneRequest asserts that the given slice contains one request, with the
// given characteristics.
func checkOneRequest(c *C, recordedRequests *[]*X509Request, URL string, payload []byte, Method string) {
    requests := *recordedRequests
    c.Assert(len(requests), Equals, 1)
    request := requests[0]
    checkRequest(c, request, URL, payload, Method)
}

func checkRequest(c *C, request *X509Request, URL string, payload []byte, Method string) {
    c.Assert(request.URL, Equals, URL)
    c.Assert(string(request.Payload), Equals, string(payload))
    c.Assert(request.Method, Equals, Method)
}

func (suite *managementBaseAPISuite) TestGetOperationFailsIfNoHeader(c *C) {
    response := x509Response{
        StatusCode: http.StatusOK,
    }

    _, err := getOperationID(&response)
    c.Check(err, ErrorMatches, ".*no operation header.*")
}

func (suite *managementBaseAPISuite) TestListOSImagesRequestsListing(c *C) {
    api := makeAPI(c)
    rigFixedResponseDispatcher(&x509Response{StatusCode: http.StatusOK, Body: []byte("<Images></Images>")})
    requests := make([]*X509Request, 0)
    rigRecordingDispatcher(&requests)

    _, err := api.ListOSImages()
    c.Assert(err, IsNil)

    c.Assert(len(requests), Equals, 1)
    c.Check(requests[0].URL, Equals, api.session.composeURL("services/images"))
}

func (suite *managementBaseAPISuite) TestListOSImagesReturnsImages(c *C) {
    expectedImage := OSImage{
        LogicalSizeInGB:   199.0,
        Label:             MakeRandomString(10),
        MediaLink:         "http://storage.example.com/" + MakeRandomString(10),
        Name:              MakeRandomString(10),
        OS:                "Linux",
        EULA:              "http://eula.example.com/" + MakeRandomString(10),
        Description:       MakeRandomString(10),
        RecommendedVMSize: "Medium",
    }
    body := fmt.Sprintf(`
        <Images>
            <OSImage>
                <LogicalSizeInGB>%f</LogicalSizeInGB>
                <Label>%s</Label>
                <MediaLink>%s</MediaLink>
                <Name>%s</Name>
                <OS>%s</OS>
                <Eula>%s</Eula>
                <Description>%s</Description>
                <RecommendedVMSize>%s</RecommendedVMSize>
            </OSImage>
        </Images>
    `,
        expectedImage.LogicalSizeInGB, expectedImage.Label,
        expectedImage.MediaLink, expectedImage.Name, expectedImage.OS,
        expectedImage.EULA, expectedImage.Description,
        expectedImage.RecommendedVMSize)
    api := makeAPI(c)
    rigFixedResponseDispatcher(&x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte(body),
    })

    listing, err := api.ListOSImages()
    c.Assert(err, IsNil)

    c.Assert(len(listing.Images), Equals, 1)
    c.Check(listing.Images[0], DeepEquals, expectedImage)
}

func (suite *managementBaseAPISuite) TestListOSImagesPreservesOrdering(c *C) {
    imageNames := []string{
        MakeRandomString(5),
        MakeRandomString(5),
        MakeRandomString(5),
    }
    body := fmt.Sprintf(`
            <Images>
                <OSImage><Name>%s</Name></OSImage>
                <OSImage><Name>%s</Name></OSImage>
                <OSImage><Name>%s</Name></OSImage>
            </Images>
        `,
        imageNames[0], imageNames[1], imageNames[2])
    api := makeAPI(c)
    rigFixedResponseDispatcher(&x509Response{StatusCode: http.StatusOK, Body: []byte(body)})

    listing, err := api.ListOSImages()
    c.Assert(err, IsNil)

    c.Assert(len(listing.Images), Equals, 3)
    receivedNames := make([]string, 3)
    for index := range listing.Images {
        receivedNames[index] = listing.Images[index].Name
    }
    c.Check(receivedNames, DeepEquals, imageNames)
}

func (suite *managementBaseAPISuite) TestListHostedServices(c *C) {
    api := makeAPI(c)
    fixedResponse := x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte("<HostedServices></HostedServices>"),
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)

    _, err := api.ListHostedServices()

    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices"
    checkOneRequest(c, &recordedRequests, expectedURL, []byte{}, "GET")
}

// TODO test that ListHostedServices parses the structures correctly

func (suite *managementBaseAPISuite) TestGetHostedServiceProperties_withoutDetails(c *C) {
    api := makeAPI(c)
    body := `
        <?xml version="1.0" encoding="utf-8"?>
        <HostedService xmlns="http://schemas.microsoft.com/windowsazure">
          <Url>hosted-service-url</Url>
          <ServiceName>hosted-service-name</ServiceName>
          <HostedServiceProperties>
            <Description>description</Description>
            <AffinityGroup>name-of-affinity-group</AffinityGroup>
            <Location>location-of-service</Location >
            <Label>base-64-encoded-name-of-service</Label>
            <Status>current-status-of-service</Status>
            <DateCreated>creation-date-of-service</DateCreated>
            <DateLastModified>last-modification-date-of-service</DateLastModified>
            <ExtendedProperties>
              <ExtendedProperty>
                <Name>name-of-property</Name>
                <Value>value-of-property</Value>
              </ExtendedProperty>
            </ExtendedProperties>
          </HostedServiceProperties>
        </HostedService>
        `
    fixedResponse := x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte(body),
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)

    serviceName := "serviceName"

    properties, err := api.GetHostedServiceProperties(serviceName, false)
    c.Assert(err, IsNil)

    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + serviceName + "?embed-detail=false"
    checkOneRequest(c, &recordedRequests, expectedURL, []byte{}, "GET")

    c.Check(properties.URL, Equals, "hosted-service-url")
    c.Check(properties.ServiceName, Equals, "hosted-service-name")
    c.Check(properties.Description, Equals, "description")
    // Details were explicitly not requested, so this is empty.
    c.Check(len(properties.Deployments), Equals, 0)
}

func (suite *managementBaseAPISuite) TestGetHostedServiceProperties_withDetails(c *C) {
    api := makeAPI(c)
    body := `
        <?xml version="1.0" encoding="utf-8"?>
        <HostedService xmlns="http://schemas.microsoft.com/windowsazure">
          <Url>hosted-service-url</Url>
          <ServiceName>hosted-service-name</ServiceName>
          <HostedServiceProperties>
            <Description>description-of-service</Description>
            <AffinityGroup>name-of-affinity-group</AffinityGroup>
            <Location>location-of-service</Location>
            <Label>base-64-encoded-name-of-service</Label>
            <Status>current-status-of-service</Status>
            <DateCreated>creation-date-of-service</DateCreated>
            <DateLastModified>last-modification-date-of-service</DateLastModified>
            <ExtendedProperties>
              <ExtendedProperty>
                <Name>name-of-property</Name>
                <Value>value-of-property</Value>
              </ExtendedProperty>
            </ExtendedProperties>
          </HostedServiceProperties>
          <Deployments>
            <Deployment xmlns="http://schemas.microsoft.com/windowsazure">
              <Name>name-of-deployment</Name>
            </Deployment>
          </Deployments>
        </HostedService>
        `
    fixedResponse := x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte(body),
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)

    serviceName := "serviceName"

    properties, err := api.GetHostedServiceProperties(serviceName, true)
    c.Assert(err, IsNil)

    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + serviceName + "?embed-detail=true"
    checkOneRequest(c, &recordedRequests, expectedURL, []byte{}, "GET")

    c.Check(properties.URL, Equals, "hosted-service-url")
    c.Check(properties.ServiceName, Equals, "hosted-service-name")
    c.Check(properties.Description, Equals, "description-of-service")
    c.Check(len(properties.Deployments), Equals, 1)
    c.Check(properties.Deployments[0].Name, Equals, "name-of-deployment")
}

func (suite *managementBaseAPISuite) TestAddHostedService(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    createHostedService := NewCreateHostedServiceWithLocation("testName", "testLabel", "East US")
    err := api.AddHostedService(createHostedService)
    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices"
    expectedPayload, err := marshalXML(createHostedService)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestDeleteHostedService(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    hostedServiceName := "testName"
    err := api.DeleteHostedService(hostedServiceName)

    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + hostedServiceName
    checkOneRequest(c, recordedRequests, expectedURL, []byte{}, "DELETE")
}

func (suite *managementBaseAPISuite) TestAddDeployment(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    serviceName := "serviceName"
    configurationSet := NewLinuxProvisioningConfiguration("testHostname12345", "test", "test123#@!", false)
    vhd := NewOSVirtualHardDisk("hostCaching", "diskLabel", "diskName", "http://mediaLink", "sourceImageName", "os")
    role := NewRole("ExtraSmall", "test-role-123", []LinuxProvisioningConfiguration{*configurationSet}, []OSVirtualHardDisk{*vhd})
    deployment := NewDeploymentForCreateVMDeployment("test-machine-name", "Staging", "testLabel", []Role{*role}, "testNetwork")
    err := api.AddDeployment(deployment, serviceName)

    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + serviceName + "/deployments"
    expectedPayload, err := marshalXML(deployment)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestDeleteDeployment(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    hostedServiceName := "testHosterServiceName"
    deploymentName := "testDeploymentName"
    err := api.DeleteDeployment(hostedServiceName, deploymentName)

    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + hostedServiceName + "/deployments/" + deploymentName
    checkOneRequest(c, recordedRequests, expectedURL, []byte{}, "DELETE")
}

var getDeploymentResponse = `
<?xml version="1.0"?>
<Deployment xmlns="http://schemas.microsoft.com/windowsazure" xmlns:i="http://www.w3.org/2001/XMLSchema-instance">
  <Name>gwaclmachinekjn8minr</Name>
  <DeploymentSlot>Staging</DeploymentSlot>
  <PrivateID>53b117c3126a4f1b8b23bc36c2c94dd1</PrivateID>
  <Status>Running</Status>
  <Label>WjNkaFkyeHRZV05vYVc1bGEycHVPRzFwYm5JPQ==</Label>
  <Url>http://53b117c3126a4f1b8b23bc36c2c94dd1.cloudapp.net/</Url>
  <Configuration>PFNlcnZpY2VDb25maWd1cmF0aW9uIHhtbG5zOnhzZD0iaHR0cDovL3d3dy53
My5vcmcvMjAwMS9YTUxTY2hlbWEiIHhtbG5zOnhzaT0iaHR0cDovL3d3dy53My5vcmcvMjAwMS9YTU
xTY2hlbWEtaW5zdGFuY2UiIHhtbG5zPSJodHRwOi8vc2NoZW1hcy5taWNyb3NvZnQuY29tL1NlcnZp
Y2VIb3N0aW5nLzIwMDgvMTAvU2VydmljZUNvbmZpZ3VyYXRpb24iPg0KICA8Um9sZSBuYW1lPSJnd2
FjbHJvbGVoYXVxODFyIj4NCiAgICA8SW5zdGFuY2VzIGNvdW50PSIxIiAvPg0KICA8L1JvbGU+DQo8
L1NlcnZpY2VDb25maWd1cmF0aW9uPg==</Configuration>
  <RoleInstanceList>
    <RoleInstance>
      <RoleName>gwaclrolehauq81r</RoleName>
      <InstanceName>gwaclrolehauq81r</InstanceName>
      <InstanceStatus>ReadyRole</InstanceStatus>
      <InstanceUpgradeDomain>0</InstanceUpgradeDomain>
      <InstanceFaultDomain>0</InstanceFaultDomain>
      <InstanceSize>ExtraSmall</InstanceSize>
      <InstanceStateDetails/>
      <IpAddress>10.241.158.13</IpAddress>
      <PowerState>Started</PowerState>
      <HostName>gwaclhostsnx7m1co57n</HostName>
      <RemoteAccessCertificateThumbprint>68db67cd8a6047a6cf6da0f92a7ee67d</RemoteAccessCertificateThumbprint>
    </RoleInstance>
  </RoleInstanceList>
  <UpgradeDomainCount>1</UpgradeDomainCount>
  <RoleList>
    <Role i:type="PersistentVMRole">
      <RoleName>gwaclrolehauq81r</RoleName>
      <OsVersion/>
      <RoleType>PersistentVMRole</RoleType>
      <ConfigurationSets>
        <ConfigurationSet i:type="NetworkConfigurationSet">
          <ConfigurationSetType>NetworkConfiguration</ConfigurationSetType>
          <SubnetNames/>
        </ConfigurationSet>
      </ConfigurationSets>
      <DataVirtualHardDisks/>
      <OSVirtualHardDisk>
        <HostCaching>ReadWrite</HostCaching>
        <DiskLabel>gwaclauonntmontirrz9rgltt8d5f4evtjeagbcx7kf8umqhs3t421m21t798ebw</DiskLabel>
        <DiskName>gwacldiskdvmvahc</DiskName>
        <MediaLink>http://gwacl3133mh3fs9jck6yk0dh.blob.core.windows.net/vhds/gwacldisk79vobmh.vhd</MediaLink>
        <SourceImageName>b39f27a8b8c64d52b05eac6a62ebad85__Ubuntu_DAILY_BUILD-precise-12_04_2-LTS-amd64-server-20130624-en-us-30GB</SourceImageName>
        <OS>Linux</OS>
      </OSVirtualHardDisk>
      <RoleSize>ExtraSmall</RoleSize>
    </Role>
  </RoleList>
  <SdkVersion/>
  <Locked>false</Locked>
  <RollbackAllowed>false</RollbackAllowed>
  <CreatedTime>2013-06-25T14:35:22Z</CreatedTime>
  <LastModifiedTime>2013-06-25T14:48:54Z</LastModifiedTime>
  <ExtendedProperties/>
  <PersistentVMDowntime>
    <StartTime>2013-05-08T22:00:00Z</StartTime>
    <EndTime>2013-05-10T06:00:00Z</EndTime>
    <Status>PersistentVMUpdateCompleted</Status>
  </PersistentVMDowntime>
  <VirtualIPs>
    <VirtualIP>
      <Address>137.117.72.69</Address>
      <IsDnsProgrammed>true</IsDnsProgrammed>
      <Name>__PseudoBackEndContractVip</Name>
    </VirtualIP>
  </VirtualIPs>
</Deployment>
`

func (suite *managementBaseAPISuite) TestGetDeployment(c *C) {
    api := makeAPI(c)
    fixedResponse := x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte(getDeploymentResponse),
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)

    serviceName := "serviceName"
    deploymentName := "gwaclmachinekjn8minr"

    request := &GetDeploymentRequest{ServiceName: serviceName, DeploymentName: deploymentName}
    deployment, err := api.GetDeployment(request)
    c.Assert(err, IsNil)

    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + serviceName + "/deployments/" + deploymentName
    checkOneRequest(c, &recordedRequests, expectedURL, []byte{}, "GET")

    c.Check(deployment.Name, Equals, deploymentName)
}

func (suite *managementBaseAPISuite) TestAddStorageAccount(c *C) {
    api := makeAPI(c)
    header := http.Header{}
    header.Set("X-Ms-Request-Id", "operationID")
    fixedResponse := x509Response{
        StatusCode: http.StatusAccepted,
        Header:     header,
    }
    rigFixedResponseDispatcher(&fixedResponse)
    recordedRequests := make([]*X509Request, 0)
    rigRecordingDispatcher(&recordedRequests)
    cssi := NewCreateStorageServiceInputWithLocation("name", "label", "East US")

    err := api.AddStorageAccount(cssi)
    c.Assert(err, IsNil)

    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/storageservices"
    expectedPayload, err := marshalXML(cssi)
    c.Assert(err, IsNil)
    checkOneRequest(c, &recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestDeleteStorageAccount(c *C) {
    const accountName = "myaccount"
    api := makeAPI(c)
    accountURL := api.session.composeURL("services/storageservices/" + accountName)
    recordedRequests := setUpDispatcher("operationID")

    err := api.DeleteStorageAccount(accountName)
    c.Assert(err, IsNil)

    checkOneRequest(c, recordedRequests, accountURL, nil, "DELETE")
}

func (suite *managementBaseAPISuite) TestGetStorageAccountKeys(c *C) {
    const (
        accountName  = "accountname"
        primaryKey   = "primarykey"
        secondaryKey = "secondarykey"
    )
    api := makeAPI(c)
    url := api.session.composeURL("services/storageservices/" + accountName)
    body := fmt.Sprintf(
        `<StorageService>
            <Url>%s</Url>
            <StorageServiceKeys>
                <Primary>%s</Primary>
                <Secondary>%s</Secondary>
            </StorageServiceKeys>
        </StorageService>`,
        url, primaryKey, secondaryKey)
    rigFixedResponseDispatcher(&x509Response{
        StatusCode: http.StatusOK,
        Body:       []byte(body),
    })

    keys, err := api.GetStorageAccountKeys("account")
    c.Assert(err, IsNil)

    c.Check(keys.Primary, Equals, primaryKey)
    c.Check(keys.Secondary, Equals, secondaryKey)
    c.Check(keys.URL, Equals, url)
}

func (suite *managementBaseAPISuite) TestDeleteDisk(c *C) {
    // The current implementation of DeleteDisk() works around a bug in
    // Windows Azure by polling the server.  See the documentation in the file
    // deletedisk.go for details.
    // Change the polling interval to speed up the tests:
    deleteDiskInterval = time.Nanosecond
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    diskName := "diskName"
    err := api.DeleteDisk(diskName)

    c.Assert(err, IsNil)
    expectedURL := api.session.composeURL("services/disks/" + diskName)
    checkOneRequest(c, recordedRequests, expectedURL, []byte{}, "DELETE")
}

func (suite *managementBaseAPISuite) TestPerformNodeOperation(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    serviceName := "serviceName"
    deploymentName := "deploymentName"
    roleName := "roleName"
    operation := newRoleOperation("RandomOperation")
    err := api.performRoleOperation(serviceName, deploymentName, roleName, operation)

    c.Assert(err, IsNil)
    expectedURL := AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" + serviceName + "/deployments/" + deploymentName + "/roleinstances/" + roleName + "/Operations"
    expectedPayload, err := marshalXML(operation)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestStartRole(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    request := &StartRoleRequest{"serviceName", "deploymentName", "roleName"}
    err := api.StartRole(request)
    c.Assert(err, IsNil)
    expectedURL := (AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" +
        request.ServiceName + "/deployments/" + request.DeploymentName + "/roleinstances/" +
        request.RoleName + "/Operations")
    expectedPayload, err := marshalXML(startRoleOperation)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestRestartRole(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    request := &RestartRoleRequest{"serviceName", "deploymentName", "roleName"}
    err := api.RestartRole(request)
    c.Assert(err, IsNil)
    expectedURL := (AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" +
        request.ServiceName + "/deployments/" + request.DeploymentName + "/roleinstances/" +
        request.RoleName + "/Operations")
    expectedPayload, err := marshalXML(restartRoleOperation)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}

func (suite *managementBaseAPISuite) TestShutdownRole(c *C) {
    api := makeAPI(c)
    recordedRequests := setUpDispatcher("operationID")
    request := &ShutdownRoleRequest{"serviceName", "deploymentName", "roleName"}
    err := api.ShutdownRole(request)
    c.Assert(err, IsNil)
    expectedURL := (AZURE_URL + api.session.subscriptionId + "/services/hostedservices/" +
        request.ServiceName + "/deployments/" + request.DeploymentName + "/roleinstances/" +
        request.RoleName + "/Operations")
    expectedPayload, err := marshalXML(shutdownRoleOperation)
    c.Assert(err, IsNil)
    checkOneRequest(c, recordedRequests, expectedURL, expectedPayload, "POST")
}
