// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "fmt"
    . "launchpad.net/gocheck"
    "net/http"
)

type managementAPISuite struct{}

var _ = Suite(&managementAPISuite{})

// makeNamedRoleInstances creates an array of RoleInstance objects, each with
// the respective given name.
func makeNamedRoleInstances(names ...string) []RoleInstance {
    instances := make([]RoleInstance, 0)
    for _, name := range names {
        instances = append(instances, RoleInstance{RoleName: name})
    }
    return instances
}

// makeNamedDeployments creates an array of Deployment objects, each with
// the respective given name.
func makeNamedDeployments(names ...string) []Deployment {
    deployments := make([]Deployment, 0)
    for _, name := range names {
        deployments = append(deployments, Deployment{Name: name})
    }
    return deployments
}

// makeHostedService creates a HostedService with the given deployments.
func makeHostedService(deployments []Deployment) HostedService {
    return HostedService{
        ServiceName: "S1",
        Deployments: deployments,
    }
}

// makeOKXMLResponse creates a DispatcherResponse with status code OK, and
// an XML-serialized version of the given object.
// The response is wrapped in a slice because that's slightly easier for
// the callers.
func makeOKXMLResponse(c *C, bodyObject AzureObject) []DispatcherResponse {
    body, err := bodyObject.Serialize()
    c.Assert(err, IsNil)
    return []DispatcherResponse{
        {
            response: &x509Response{
                StatusCode: http.StatusOK,
                Body:       []byte(body),
            },
        },
    }
}

// TestListInstances goes through the happy path for ListInstances.
func (suite *managementAPISuite) TestListInstances(c *C) {
    service := makeHostedService(
        []Deployment{
            {RoleInstanceList: makeNamedRoleInstances("one", "two")},
            {RoleInstanceList: makeNamedRoleInstances("three", "four")},
        })
    record := []*X509Request{}
    rigRecordingPreparedResponseDispatcher(&record, makeOKXMLResponse(c, service))

    // Exercise ListInstances.
    api := makeAPI(c)
    request := &ListInstancesRequest{ServiceName: service.ServiceName}
    instances, err := api.ListInstances(request)
    c.Assert(err, IsNil)

    // We get the expected instances back.
    c.Check(instances, DeepEquals, []RoleInstance{
        service.Deployments[0].RoleInstanceList[0],
        service.Deployments[0].RoleInstanceList[1],
        service.Deployments[1].RoleInstanceList[0],
        service.Deployments[1].RoleInstanceList[1],
    })

    // The only request is for the service's properties
    c.Assert(len(record), Not(Equals), 0)
    expectedURL := fmt.Sprintf(
        "%ssubscriptionId/services/hostedservices/%s?embed-detail=true",
        AZURE_URL, service.ServiceName)
    c.Check(record[0], DeepEquals, &X509Request{
        APIVersion: baseAPIVersion,
        URL:        expectedURL,
        Method:     "GET",
    })
}

func (suite *managementAPISuite) TestListInstancesFailsGettingDetails(c *C) {
    rigPreparedResponseDispatcher(
        []DispatcherResponse{{response: &x509Response{StatusCode: http.StatusNotFound}}},
    )
    api := makeAPI(c)
    request := &ListInstancesRequest{ServiceName: "SomeService"}
    instances, err := api.ListInstances(request)
    c.Check(instances, IsNil)
    c.Assert(err, NotNil)
    c.Assert(err.Error(), Equals, "GET request failed (404: Not Found)")
}

// TestListAllDeployments goes through the happy path for ListAllDeployments.
func (suite *managementAPISuite) TestListAllDeployments(c *C) {
    service := makeHostedService(makeNamedDeployments("one", "two"))
    record := []*X509Request{}
    rigRecordingPreparedResponseDispatcher(&record, makeOKXMLResponse(c, service))

    // Exercise ListDeployments.
    api := makeAPI(c)
    request := &ListAllDeploymentsRequest{ServiceName: service.ServiceName}
    deployments, err := api.ListAllDeployments(request)
    c.Assert(err, IsNil)

    // We get the complete set of deployments back.
    c.Check(deployments, DeepEquals, service.Deployments)

    // Only one request to the API is made.
    c.Assert(len(record), Equals, 1)
}

// TestListDeployments tests ListDeployments, including filtering by name.
func (suite *managementAPISuite) TestListDeployments(c *C) {
    service := makeHostedService(makeNamedDeployments("Arthur", "Bobby"))
    record := []*X509Request{}
    rigRecordingPreparedResponseDispatcher(&record, makeOKXMLResponse(c, service))

    // Exercise ListDeployments.
    api := makeAPI(c)
    request := &ListDeploymentsRequest{
        ServiceName:     service.ServiceName,
        DeploymentNames: []string{"Arthur"},
    }
    deployments, err := api.ListDeployments(request)
    c.Assert(err, IsNil)

    // Only the first deployment - named "Arthur" - is returned.
    c.Check(deployments, DeepEquals, []Deployment{service.Deployments[0]})
    // Only one request to the API is made.
    c.Assert(len(record), Equals, 1)
}

func (suite *managementAPISuite) TestListDeploymentsWithoutNamesReturnsNothing(c *C) {
    service := makeHostedService(makeNamedDeployments("One", "Two"))
    record := []*X509Request{}
    rigRecordingPreparedResponseDispatcher(&record, makeOKXMLResponse(c, service))
    // Exercise ListDeployments.
    api := makeAPI(c)
    request := &ListDeploymentsRequest{
        ServiceName:     service.ServiceName,
        DeploymentNames: []string{},
    }
    deployments, err := api.ListDeployments(request)
    c.Assert(err, IsNil)

    // No deployments are returned.
    c.Check(len(deployments), Equals, 0)
}
