package gwacl

import (
    "bufio"
    "bytes"
    "fmt"
    curl "github.com/andelf/go-curl"
    . "launchpad.net/gwacl/logging"
    "net/http"
    "net/textproto"
)

type X509Request struct {
    APIVersion  string
    Method      string
    URL         string
    Payload     []byte
    ContentType string
}

// Print debugging output.
var verbose = false

func SetVerbose(newVerbose bool) {
    verbose = newVerbose
}

// baseAPIVersion is the default Azure API version to use.
const baseAPIVersion = "2013-03-01"

// newX509RequestGET initializes an X509Request for a GET.  You may still need
// to set further values.
func newX509RequestGET(url string) *X509Request {
    return &X509Request{
        Method:     "GET",
        URL:        url,
        APIVersion: baseAPIVersion,
    }
}

// newX509RequestPOST initializes an X509Request for a POST.  You may still
// need to set further values.
func newX509RequestPOST(url string, payload []byte, contentType string) *X509Request {
    return &X509Request{
        Method:      "POST",
        URL:         url,
        APIVersion:  baseAPIVersion,
        Payload:     payload,
        ContentType: contentType,
    }
}

// newX509RequestDELETE initializes an X509Request for a DELETE.
func newX509RequestDELETE(url string) *X509Request {
    return &X509Request{
        Method:     "DELETE",
        URL:        url,
        APIVersion: baseAPIVersion,
    }
}

// newX509RequestPUT initializes an X509Request for a PUT.  You may still
// need to set further values.
func newX509RequestPUT(url string, payload []byte) *X509Request {
    return &X509Request{
        Method:     "PUT",
        URL:        url,
        APIVersion: baseAPIVersion,
        Payload:    payload,
    }
}

type x509Response struct {
    StatusCode int
    // TODO: What exactly do we get back?  How will we know its encoding?
    Body      []byte
    RawHeader []byte
    Header    http.Header
}

func newX509Response() *x509Response {
    return &x509Response{
        Body:      make([]byte, 0),
        RawHeader: make([]byte, 0),
    }
}

// HTTP/MS-DOS-style line break.
const crlf = "\r\n"

// stripPreliminaryResponses takes an HTTP response and returns the final
// response block.  It leaves out any preceding "Continue" responses or
// other non-final responses.
func stripPreliminaryResponses(httpResponse []byte) []byte {
    blankLine := []byte(crlf + crlf)

    // The response ends in a blank line, although we don't want to rely on
    // that without clear documentation of go-curl's header-writing API.
    // Ignoring that final optional one, where is the last blank line?
    withoutLastBlank := httpResponse[:len(httpResponse)-len(blankLine)]
    lastBlank := bytes.LastIndex(withoutLastBlank, blankLine)
    if lastBlank == -1 {
        // There are no preliminary responses.  The response is fine as-is.
        return httpResponse
    }
    return httpResponse[lastBlank+len(blankLine):]
}

// stripResponseLine takes an HTTP response and returns the part that makes
// up the MIME header.
func stripResponseLine(httpHeader []byte) ([]byte, error) {
    lineBreak := []byte(crlf)
    newlinePos := bytes.Index(httpHeader, lineBreak)
    if newlinePos == -1 {
        msg := fmt.Errorf("HTTP response header lacks leading line: '%s'", httpHeader)
        return nil, msg
    }
    return httpHeader[newlinePos+len(lineBreak):], nil
}

// parseHeader parses the raw header as stored in RawHeader, and uses it to
// initialize Header.
func (response *x509Response) parseHeader() error {
    rawHeader := stripPreliminaryResponses(response.RawHeader)
    mimeHeader, err := stripResponseLine(rawHeader)
    if err != nil {
        return err
    }

    // textproto.NewReader() parses a MIME header.  But it requires a
    // bufio.Reader (struct), not an io.Reader (interface).  So wrap the header
    // contents in an io.Reader using bytes.NewReader(), then wrap that in a
    // bufio.Reader using bufio.NewReader(), and then pass that to
    // textproto.NewReader().
    reader := textproto.NewReader(bufio.NewReader(bytes.NewReader(mimeHeader)))
    header, err := reader.ReadMIMEHeader()
    if err != nil {
        return err
    }
    response.Header = http.Header(header)
    return nil
}

func performX509CurlRequest(session *x509Session, request *X509Request) (*x509Response, error) {
    if verbose {
        Debug("Performing request")
        Debug("Request url: " + request.URL)
        Debug("Request method: " + request.Method)
        Debugf("Request body: %s\n", request.Payload)
    }
    response := newX509Response()
    ch := request.makeCurlRequest(session, response)
    defer ch.Cleanup()

    // TODO: Add timeout provision.
    // TODO: Get rid of the annoying debug message go-curl spits out
    // for every single request: "debug (Getinfo) 200".
    err := ch.Perform()
    if err != nil {
        return nil, err
    }

    status, err := ch.Getinfo(curl.INFO_RESPONSE_CODE)
    if err != nil {
        return nil, err
    }
    err = response.parseHeader()
    if err != nil {
        return nil, err
    }
    response.StatusCode = status.(int)
    if verbose {
        Debug("Got response")
        Debugf("Response status: %d\n", response.StatusCode)
        Debugf("Response headers: %s\n", response.RawHeader)
        Debugf("Response body: %s\n", response.Body)
    }
    return response, nil
}

// makeCurlRequest produces a curl.CURL representing the request.
// Clean up the request by calling its Cleanup() method after you're done
// with it.
func (request *X509Request) makeCurlRequest(session *x509Session, response *x509Response) *curl.CURL {
    ch := curl.EasyInit()

    ch.Setopt(curl.OPT_SSLCERT, session.certFile)
    ch.Setopt(curl.OPT_KEYPASSWD, "")
    ch.Setopt(curl.OPT_URL, request.URL)
    //ch.Setopt(curl.OPT_VERBOSE, true)
    ch.Setopt(curl.OPT_SSLVERSION, 3)
    versionHeader := "x-ms-version: " + request.APIVersion
    headers := []string{versionHeader}
    request.setCurlHTTPMethod(ch)

    if len(request.Payload) != 0 {
        headers = append(headers, "Content-Type: "+request.ContentType)
        // Arrange for the request body to be written.
        ch.Setopt(curl.OPT_POSTFIELDSIZE, len(request.Payload))
        ch.Setopt(curl.OPT_READFUNCTION, func(buf []byte, _ interface{}) int {
            // Buffered request write.  Copy as much of the payload as will
            // fit into the buffer curl gave us for it, and return the number
            // of bytes written.  The calls to this callback will stop once
            // we return zero.
            // TODO: Test this against various nasty data sizes.
            bytesCopied := copy(buf, request.Payload)
            request.Payload = request.Payload[bytesCopied:]
            return bytesCopied
        })
    }

    // Arrange for the response body to be stored in response.Body.
    ch.Setopt(curl.OPT_WRITEFUNCTION, func(data []byte, _ interface{}) bool {
        response.Body = append(response.Body, data...)
        // Indicate that we're done processing the data.
        return true
    })

    // Arrange for the response header to be stored in response.RawHeader.
    // After the request we can parse them into response.Header.
    ch.Setopt(curl.OPT_HEADERFUNCTION, func(data []byte, _ interface{}) bool {
        response.RawHeader = append(response.RawHeader, data...)
        // Indicate that we're done processing the data.
        return true
    })

    ch.Setopt(curl.OPT_HTTPHEADER, headers)
    return ch
}

// setCurlHTTPMethod sets up a CURL object to perform an HTTP request of
// the HTTP method (GET, POST, etc.) appropriate for the request.
func (request *X509Request) setCurlHTTPMethod(ch *curl.CURL) {
    switch request.Method {
    case "GET":
        // Nothing to be done.  This is the default.
    case "POST":
        ch.Setopt(curl.OPT_POST, true)
    case "PUT":
        ch.Setopt(curl.OPT_CUSTOMREQUEST, "PUT")
    case "DELETE":
        ch.Setopt(curl.OPT_CUSTOMREQUEST, "DELETE")
    default:
        panic(fmt.Errorf("Unsupported http method: %s", request.Method))
    }
}
