package gwacl

import (
    . "launchpad.net/gocheck"
    "net/http"
    "net/http/httptest"
)

type x509DispatcherSuite struct{}

var _ = Suite(&x509DispatcherSuite{})

// makeRecordingHTTPServer creates an http server (don't forget to Close() it when done)
// that serves at the given base URL, copies incoming requests into the given
// channel, and finally returns the given status code.  If body is not nil, it
// will be returned as the request body.
func makeRecordingHTTPServer(requests chan *http.Request, status int, body []byte) *httptest.Server {
    returnRequest := func(w http.ResponseWriter, r *http.Request) {
        requests <- r
        w.WriteHeader(status)
        if body != nil {
            w.Write(body)
        }
    }
    serveMux := http.NewServeMux()
    serveMux.HandleFunc("/", returnRequest)
    return httptest.NewServer(serveMux)
}

// An HTTP header contains a response line followed by a MIME header, which
// parseHeader processes.
func (*x509DispatcherSuite) TestParseHeaderParsesHTTPHeader(c *C) {
    response := x509Response{}
    text := "HTTP/1.1 200 OK\r\nCache-Control: no-cache\r\n\r\n"
    response.RawHeader = []byte(text)

    err := response.parseHeader()
    c.Assert(err, IsNil)

    c.Check(response.Header["Cache-Control"], DeepEquals, []string{"no-cache"})
}

// An HTTP response actually contains a sequence of HTTP headers, and
// parseHeader only uses the final one.
func (*x509DispatcherSuite) TestParseHeaderParsesContinueResponse(c *C) {
    response := x509Response{}
    text := "HTTP/1.1 100 Continue\r\n\r\nHTTP/1.1 202 Accepted\r\nCache-Control: no-cache\r\n\r\n"
    response.RawHeader = []byte(text)

    err := response.parseHeader()
    c.Assert(err, IsNil)

    c.Check(response.Header["Cache-Control"], DeepEquals, []string{"no-cache"})
}

func (*x509DispatcherSuite) TestGetRequestDoesHTTPGET(c *C) {
    httpRequests := make(chan *http.Request, 1)
    server := makeRecordingHTTPServer(httpRequests, http.StatusOK, nil)
    defer server.Close()
    // No real certificate needed since we're testing on http, not https.
    session, err := newX509Session("subscriptionid", "cert.pem")
    c.Assert(err, IsNil)
    path := "/foo/bar"
    request := newX509RequestGET(server.URL + path)

    response, err := performX509CurlRequest(session, request)
    c.Assert(err, IsNil)
    c.Assert(response.StatusCode, Equals, http.StatusOK)

    httpRequest := <-httpRequests
    c.Check(httpRequest.Method, Equals, "GET")
    c.Check(httpRequest.URL.String(), Equals, path)
}

func (*x509DispatcherSuite) TestPostRequestDoesHTTPPOST(c *C) {
    httpRequests := make(chan *http.Request, 1)
    requestBody := []byte{1, 2, 3}
    responseBody := []byte{4, 5, 6}
    requestContentType := "bogusContentType"
    server := makeRecordingHTTPServer(httpRequests, http.StatusOK, responseBody)
    defer server.Close()
    // No real certificate needed since we're testing on http, not https.
    session, err := newX509Session("subscriptionid", "cert.pem")
    c.Assert(err, IsNil)
    path := "/foo/bar"
    request := newX509RequestPOST(server.URL+path, requestBody, requestContentType)

    response, err := performX509CurlRequest(session, request)
    c.Assert(err, IsNil)
    c.Assert(response.StatusCode, Equals, http.StatusOK)

    httpRequest := <-httpRequests
    c.Check(httpRequest.Header[http.CanonicalHeaderKey("Content-Type")], DeepEquals, []string{requestContentType})
    c.Check(httpRequest.Header[http.CanonicalHeaderKey("X-Ms-Version")], DeepEquals, []string{request.APIVersion})
    c.Check(httpRequest.Method, Equals, "POST")
    c.Check(httpRequest.URL.String(), Equals, path)
    c.Check(response.Body, DeepEquals, responseBody)
}

func (*x509DispatcherSuite) TestDeleteRequestDoesHTTPDELETE(c *C) {
    httpRequests := make(chan *http.Request, 1)
    server := makeRecordingHTTPServer(httpRequests, http.StatusOK, []byte{})
    defer server.Close()
    // No real certificate needed since we're testing on http, not https.
    session, err := newX509Session("subscriptionid", "cert.pem")
    c.Assert(err, IsNil)
    path := "/foo/bar"
    request := newX509RequestDELETE(server.URL + path)

    response, err := performX509CurlRequest(session, request)
    c.Assert(err, IsNil)
    c.Assert(response.StatusCode, Equals, http.StatusOK)

    httpRequest := <-httpRequests
    c.Check(httpRequest.Method, Equals, "DELETE")
    c.Check(httpRequest.URL.String(), Equals, path)
}

func (*x509DispatcherSuite) TestPutRequestDoesHTTPPUT(c *C) {
    httpRequests := make(chan *http.Request, 1)
    requestBody := []byte{1, 2, 3}
    responseBody := []byte{4, 5, 6}
    server := makeRecordingHTTPServer(httpRequests, http.StatusOK, responseBody)
    defer server.Close()
    // No real certificate needed since we're testing on http, not https.
    session, err := newX509Session("subscriptionid", "cert.pem")
    c.Assert(err, IsNil)
    path := "/foo/bar"
    request := newX509RequestPUT(server.URL+path, requestBody)

    response, err := performX509CurlRequest(session, request)
    c.Assert(err, IsNil)
    c.Assert(response.StatusCode, Equals, http.StatusOK)

    httpRequest := <-httpRequests
    c.Check(httpRequest.Method, Equals, "PUT")
    c.Check(httpRequest.URL.String(), Equals, path)
    c.Check(response.Body, DeepEquals, responseBody)
}

func (*x509DispatcherSuite) TestRequestRegistersHeader(c *C) {
    customHeader := http.CanonicalHeaderKey("x-gwacl-test")
    customValue := []string{"present"}
    returnRequest := func(w http.ResponseWriter, r *http.Request) {
        w.Header()[customHeader] = customValue
        w.WriteHeader(http.StatusOK)
    }
    serveMux := http.NewServeMux()
    serveMux.HandleFunc("/", returnRequest)
    server := httptest.NewServer(serveMux)
    defer server.Close()
    session, err := newX509Session("subscriptionid", "cert.pem")
    c.Assert(err, IsNil)
    path := "/foo/bar"
    request := newX509RequestGET(server.URL + path)

    response, err := performX509CurlRequest(session, request)
    c.Assert(err, IsNil)

    c.Check(response.Header[customHeader], DeepEquals, customValue)
}
