// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "encoding/base64"
    "encoding/xml"
    "fmt"
    "net/url"
    "regexp"
    "sort"
    "strings"
    "time"
)

// It's impossible to have any kind of common method inherited into all the
// various serializable objects because the receiver of the method is the
// wrong class which confuses the xml marshaller.  Hence, this mess.
type AzureObject interface {
    Serialize() (string, error)
}

// marshalXML is a wrapper for serializing objects to XML in the visual layout
// that gwacl prefers.
func marshalXML(obj interface{}) ([]byte, error) {
    return xml.MarshalIndent(obj, "", "  ")
}

func toxml(obj AzureObject) (string, error) {
    out, err := marshalXML(obj)
    if err != nil {
        return "", err
    }
    return string(out), nil
}

//
// LinuxProvisioningConfiguration bits
//

// TODO: this should contain InputEndpoints objects.
type LinuxProvisioningConfiguration struct {
    ConfigurationSetType             string `xml:"ConfigurationSetType"` // "LinuxProvisioningConfiguration"
    Hostname                         string `xml:"HostName"`
    Username                         string `xml:"UserName"`
    Password                         string `xml:"UserPassword"`
    DisableSSHPasswordAuthentication bool   `xml:"DisableSshPasswordAuthentication"`
}

func (c *LinuxProvisioningConfiguration) Serialize() (string, error) {
    return toxml(c)
}

func NewLinuxProvisioningConfiguration(
    Hostname, Username, Password string,
    DisableSSHPasswordAuthentication bool) *LinuxProvisioningConfiguration {
    return &LinuxProvisioningConfiguration{
        ConfigurationSetType: "LinuxProvisioningConfiguration",
        Hostname:             Hostname,
        Username:             Username,
        Password:             Password,
        DisableSSHPasswordAuthentication: DisableSSHPasswordAuthentication,
    }
}

//
// InputEndpoint bits
//

type InputEndpoint struct {
    LoadBalancedEndpointSetName string `xml:"LoadBalancedEndpointSetName"`
    LocalPort                   int    `xml:"LocalPort"`
    Name                        string `xml:"Name"`
    Port                        int    `xml:"Port"`
    Protocol                    string `xml:"Protocol"` // TCP or UDP
}

func (c *InputEndpoint) Serialize() (string, error) {
    return toxml(c)
}

//
// Images bits
//

// Images is a series of OSImages.
type Images struct {
    Images []OSImage `xml:"OSImage"`
}

func (i *Images) Deserialize(data []byte) error {
    return xml.Unmarshal(data, i)
}

var canonicalPublisherName = "Canonical"
var imageFamilyFormatRegexp = "^Ubuntu Server %s.*$"

func (images *Images) Len() int {
    return len(images.Images)
}

func (images *Images) Swap(i, j int) {
    images.Images[i], images.Images[j] = images.Images[j], images.Images[i]
}

// Less returns true if the image at index i is newer than the one at index j, comparing by
// PublishedDate.
// This function is used by sort.Sort().
func (images *Images) Less(i, j int) bool {
    // We need to implement the sort interface so Less cannot return an error.  We panic if
    // one of the dates cannot be parse and the calling method will recover this.
    dateStringI := images.Images[i].PublishedDate
    dateI, err := time.Parse(time.RFC3339, dateStringI)
    if err != nil {
        panic(fmt.Errorf("Failed to parse image's 'PublishedDate': %s", dateStringI))
    }
    dateStringJ := images.Images[j].PublishedDate
    dateJ, err := time.Parse(time.RFC3339, dateStringJ)
    if err != nil {
        panic(fmt.Errorf("Failed to parse image's 'PublishedDate': %s", dateStringJ))
    }
    return dateI.After(dateJ)
}

// GetLatestUbuntuImage returns the most recent released available OSImage,
// for the given release name and location.  The 'releaseName' parameter is
// the Ubuntu version number present in the 'ImageFamily' tag present in 
// Azure's representation of an OS Image (e.g. '12.04', '12.10').
func (images *Images) GetLatestUbuntuImage(releaseName string, location string) (image *OSImage, err error) {
    // The Less method defined above can panic if one of the published dates cannot be parsed,
    // this code recovers from that and transforms that into an error.
    defer func() {
        if recoveredErr := recover(); recoveredErr != nil {
            image = nil
            err = recoveredErr.(error)
        }
    }()
    matcherRegexp := regexp.MustCompile(fmt.Sprintf(imageFamilyFormatRegexp, releaseName))
    matchingImages := Images{}
    for _, image := range images.Images {
        if image.PublisherName == canonicalPublisherName &&
            matcherRegexp.MatchString(image.ImageFamily) &&
            image.hasLocation(location) &&
            !image.isDailyBuild() {
            matchingImages.Images = append(matchingImages.Images, image)
        }
    }
    if matchingImages.Len() == 0 {
        return nil, fmt.Errorf("No matching images found")
    }
    sort.Sort(&matchingImages)
    return &matchingImages.Images[0], nil
}

//
// OSImage bits
//

// OSImage represents a disk image containing an operating system.
// Confusingly, the Azure API documentation also calls it a VM image.
type OSImage struct {
    AffinityGroup     string  `xml:"AffinityGroup,omitempty"`
    Category          string  `xml:"Category"`
    Label             string  `xml:"Label"`
    Location          string  `xml:"Location"`
    LogicalSizeInGB   float32 `xml:"LogicalSizeInGB"`
    MediaLink         string  `xml:"MediaLink"`
    Name              string  `xml:"Name"`
    OS                string  `xml:"OS"`
    EULA              string  `xml:"Eula,omitempty"`
    Description       string  `xml:"Description,omitempty"`
    ImageFamily       string  `xml:"ImageFamily,omitempty"`
    PublishedDate     string  `xml:"PublishedDate,omitempty"`
    IsPremium         string  `xml:"IsPremium,omitempty"`
    PrivacyURI        string  `xml:"PrivacyUri,omitempty"`
    PricingDetailLink string  `xml:"PricingDetailLink,omitempty"`
    IconURI           string  `xml:"IconUri,omitempty"`
    RecommendedVMSize string  `xml:"RecommendedVMSize,omitempty"`
    PublisherName     string  `xml:"PublisherName"`
    ShowInGUI         string  `xml:"ShowInGui"`
    SmallIconURI      string  `xml:"SmallIconUri,omitempty"`
    Language          string  `xml:"Language"`
}

func (image *OSImage) hasLocation(location string) bool {
    locations := strings.Split(image.Location, ";")
    for _, loc := range locations {
        if loc == location {
            return true
        }
    }
    return false
}

// isDailyBuild returns whether this image is a daily build.
func (image *OSImage) isDailyBuild() bool {
    return strings.Contains(image.Label, "DAILY")
}

func (i *OSImage) Deserialize(data []byte) error {
    return xml.Unmarshal(data, i)
}

//
// OSVirtualHardDisk bits
//

type HostCachingType string

const (
    HostCachingRO HostCachingType = "ReadOnly"
    HostCachingRW HostCachingType = "ReadWrite"
)

type OSVirtualHardDisk struct {
    HostCaching     string `xml:"HostCaching,omitempty"`
    DiskLabel       string `xml:"DiskLabel,omitempty"`
    DiskName        string `xml:"DiskName,omitempty"`
    MediaLink       string `xml:"MediaLink,omitempty"`
    SourceImageName string `xml:"SourceImageName,omitempty"`
    OS              string `xml:"OS,omitempty"`
}

func (c *OSVirtualHardDisk) Serialize() (string, error) {
    return toxml(c)
}

func NewOSVirtualHardDisk(
    HostCaching HostCachingType, DiskLabel, DiskName, MediaLink,
    SourceImageName, OS string) *OSVirtualHardDisk {
    return &OSVirtualHardDisk{
        HostCaching:     string(HostCaching),
        DiskLabel:       DiskLabel,
        DiskName:        DiskName,
        SourceImageName: SourceImageName,
        MediaLink:       MediaLink,
        OS:              OS,
    }
}

//
// NetworkConfiguration bits
//

type NetworkConfiguration struct {
    InputEndpoints []InputEndpoint `xml:"InputEndpoints>InputEndpoint"`
}

func (c *NetworkConfiguration) Serialize() (string, error) {
    return toxml(c)
}

type Role struct {
    RoleName          string                           `xml:"RoleName"`
    RoleType          string                           `xml:"RoleType"` // Always "PersistentVMRole"
    ConfigurationSets []LinuxProvisioningConfiguration `xml:"ConfigurationSets>ConfigurationSet"`
    OSVirtualHardDisk []OSVirtualHardDisk              `xml:"OSVirtualHardDisk"`
    RoleSize          string                           `xml:"RoleSize"`
}

//
// Role bits
//

func (c *Role) Serialize() (string, error) {
    return toxml(c)
}

func NewRole(RoleSize string, RoleName string,
    ConfigurationSets []LinuxProvisioningConfiguration, vhds []OSVirtualHardDisk) *Role {
    return &Role{
        RoleSize:          RoleSize,
        RoleName:          RoleName,
        RoleType:          "PersistentVMRole",
        ConfigurationSets: ConfigurationSets,
        OSVirtualHardDisk: vhds,
    }
}

//
// DnsServer bits
//

type DnsServer struct {
    Name    string `xml:"Name"`
    Address string `xml:"Address"`
}

func (c *DnsServer) Serialize() (string, error) {
    return toxml(c)
}

//
// Hosted service bits
//

type HostedService struct {
    XMLNS       string       `xml:"xmlns,attr"`
    URL         string       `xml:"Url"`
    ServiceName string       `xml:"ServiceName"`
    Description string       `xml:"HostedServiceProperties>Description"`
    Deployments []Deployment `xml:"Deployments>Deployment"`
}

func (c HostedService) Serialize() (string, error) {
    return toxml(c)
}

type HostedServiceDescriptorList struct {
    HostedServices []HostedServiceDescriptor `xml:"HostedService"`
}

func (c *HostedServiceDescriptorList) Serialize() (string, error) {
    return toxml(c)
}

// HostedServiceDescriptor contains a subset of the details in HostedService,
// and is used when describing a list of HostedServices.
type HostedServiceDescriptor struct {
    XMLNS       string `xml:"xmlns,attr"`
    URL         string `xml:"Url"`
    ServiceName string `xml:"ServiceName"`
    Description string `xml:"Description"`
}

type CreateHostedService struct {
    XMLNS              string             `xml:"xmlns,attr"`
    ServiceName        string             `xml:"ServiceName"`
    Label              string             `xml:"Label"` // base64-encoded
    Description        string             `xml:"Description"`
    Location           string             `xml:"Location,omitempty"`
    AffinityGroup      string             `xml:"AffinityGroup,omitempty"`
    ExtendedProperties []ExtendedProperty `xml:"ExtendedProperties>ExtendedProperty"`
}

func NewCreateHostedServiceWithLocation(serviceName, label, location string) *CreateHostedService {
    base64label := base64.StdEncoding.EncodeToString([]byte(label))
    return &CreateHostedService{
        XMLNS:       XMLNS,
        ServiceName: serviceName,
        Label:       base64label,
        Location:    location,
    }
}

func (s *CreateHostedService) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

//
// Deployment bits
//

// Deployment is used both as input for the "Create Virtual Machine Deployment"
// call, and as a return value for "Get Deployment."
type Deployment struct {
    XMLNS              string             `xml:"xmlns,attr"`
    XMLNS_I            string             `xml:"xmlns:i,attr"`
    Name               string             `xml:"Name"`
    DeploymentSlot     string             `xml:"DeploymentSlot"`
    PrivateID          string             `xml:"PrivateID,omitempty"` // Only used for "Get Deployment."
    Status             string             `xml:"Status,omitempty"`    // Only used for "Get Deployment."
    Label              string             `xml:"Label"`
    URL                string             `xml:"Url,omitempty"`           // Only used for "Get Deployment."
    Configuration      string             `xml:"Configuration,omitempty"` // Only used for "Get Deployment."
    RoleInstanceList   []RoleInstance     `xml:"RoleInstanceList>RoleInstance"`
    UpgradeDomainCount string             `xml:"UpgradeDomainCount,omitempty"` // Only used for "Get Deployment."
    RoleList           []Role             `xml:"RoleList>Role"`
    SDKVersion         string             `xml:"SdkVersion,omitempty"`      // Only used for "Get Deployment."
    Locked             string             `xml:"Locked,omitempty"`          // Only used for "Get Deployment."
    RollbackAllowed    string             `xml:"RollbackAllowed,omitempty"` // Only used for "Get Deployment."
    VirtualNetworkName string             `xml:VirtualNetworkName,omitempty"`
    DNS                []DnsServer        `xml:"Dns>DnsServers>DnsServer",omitempty`
    ExtendedProperties []ExtendedProperty `xml:"ExtendedProperties>ExtendedProperty,omitempty"` // Only used for "Get Deployment."
}

func (deployment *Deployment) GetFQDN() (string, error) {
    if deployment.URL == "" {
        return "", fmt.Errorf("Deployment's URL field is empty")
    }
    parsedURL, err := url.Parse(deployment.URL)
    if err != nil {
        return "", err
    }
    return parsedURL.Host, nil
}

func (s *Deployment) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

func (c *Deployment) Serialize() (string, error) {
    return toxml(c)
}

// RoleInstance is a component of a Deployment.
type RoleInstance struct {
    RoleName                          string             `xml:"RoleName"`
    InstanceName                      string             `xml:"InstanceName"`
    InstanceStatus                    string             `xml:"InstanceStatus"`
    InstanceUpgradeDomain             string             `xml:"InstanceUpgradeDomain"`
    InstanceFaultDomain               string             `xml:"InstanceFaultDomain"`
    InstanceSize                      string             `xml:"InstanceSize"`
    InstanceStateDetails              string             `xml:"InstanceStateDetails"`
    InstanceErrorCode                 string             `xml:"InstanceErrorCode"`
    IPAddress                         string             `xml:"IpAddress"`
    InstanceEndpoints                 []InstanceEndpoint `xml:"InstanceEndpoints>InstanceEndpoint"`
    PowerState                        string             `xml:"PowerState"`
    HostName                          string             `xml:"HostName"`
    RemoteAccessCertificateThumbprint string             `xml:"RemoteAccessCertificateThumbprint"`
}

// InstanceEndpoint is a component of a RoleInstance.
type InstanceEndpoint struct {
    Name       string `xml:"Name"`
    VIP        string `xml:"Vip"`
    PublicPort uint16 `xml:"PublicPort"`
    LocalPort  uint16 `xml:"LocalPort"`
    Protocol   string `xml:"Protocol"`
}

// newDeploymentForCreateVMDeployment creates a Deployment object for the
// purpose of passing it to "Create Virtual Machine Deployment."
// You may still want to set the optional DNS attribute.
func NewDeploymentForCreateVMDeployment(name, deploymentSlot, label string, roles []Role, virtualNetworkName string) *Deployment {
    deployment := Deployment{
        XMLNS:              XMLNS,
        XMLNS_I:            XMLNS_I,
        Name:               name,
        DeploymentSlot:     deploymentSlot,
        Label:              base64.StdEncoding.EncodeToString([]byte(label)),
        RoleList:           roles,
        VirtualNetworkName: virtualNetworkName,
    }
    return &deployment
}

const XMLNS = "http://schemas.microsoft.com/windowsazure"
const XMLNS_I = "http://www.w3.org/2001/XMLSchema-instance"

//
// Role Operations bits
//

type RoleOperation struct {
    XMLName       xml.Name
    XMLNS         string `xml:"xmlns,attr"`
    XMLNS_I       string `xml:"xmlns:i,attr"`
    OperationType string `xml:"OperationType"`
}

func newRoleOperation(operationType string) *RoleOperation {
    operation := RoleOperation{
        XMLNS:         XMLNS,
        XMLNS_I:       XMLNS_I,
        OperationType: operationType,
    }
    operation.XMLName.Local = operationType
    return &operation
}

// The Start Role operation starts a virtual machine. 
// http://msdn.microsoft.com/en-us/library/windowsazure/jj157189.aspx
var startRoleOperation = newRoleOperation("StartRoleOperation")

// The Shutdown Role operation shuts down a virtual machine.
// http://msdn.microsoft.com/en-us/library/windowsazure/jj157195.aspx
var shutdownRoleOperation = newRoleOperation("ShutdownRoleOperation")

// The Restart role operation restarts a virtual machine.
// http://msdn.microsoft.com/en-us/library/windowsazure/jj157197.aspx
var restartRoleOperation = newRoleOperation("RestartRoleOperation")

//
// Storage Services bits
//

type ExtendedProperty struct {
    Name  string `xml:"Name"`
    Value string `xml:"Value"`
}

type StorageService struct {
    // List Storage Accounts.
    // See http://msdn.microsoft.com/en-us/library/windowsazure/ee460787.aspx
    URL                   string             `xml:"Url"`
    ServiceName           string             `xml:"ServiceName"`
    Description           string             `xml:"StorageServiceProperties>Description"`
    AffinityGroup         string             `xml:"StorageServiceProperties>AffinityGroup"`
    Label                 string             `xml:"StorageServiceProperties>Label"` // base64
    Status                string             `xml:"StorageServiceProperties>Status"`
    Endpoints             []string           `xml:"StorageServiceProperties>Endpoints>Endpoint"`
    GeoReplicationEnabled string             `xml:"StorageServiceProperties>GeoReplicationEnabled"`
    GeoPrimaryRegion      string             `xml:"StorageServiceProperties>GeoPrimaryRegion"`
    StatusOfPrimary       string             `xml:"StorageServiceProperties>StatusOfPrimary"`
    LastGeoFailoverTime   string             `xml:"StorageServiceProperties>LastGeoFailoverTime"`
    GeoSecondaryRegion    string             `xml:"StorageServiceProperties>GeoSecondaryRegion"`
    StatusOfSecondary     string             `xml:"StorageServiceProperties>StatusOfSecondary"`
    ExtendedProperties    []ExtendedProperty `xml:"StorageServiceProperties>ExtendedProperties>ExtendedProperty"`

    // TODO: Add accessors for non-string data encoded as strings.
}

type StorageServices struct {
    XMLNS           string           `xml:"xmlns,attr"`
    StorageServices []StorageService `xml:"StorageService"`
}

func (s *StorageServices) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

// CreateStorageServiceInput is a request to create a storage account.
// (Azure's "storage services" seem to have been renamed to "storage accounts"
// but the old terminology is still evident in the API).
type CreateStorageServiceInput struct {
    // See http://msdn.microsoft.com/en-us/library/windowsazure/hh264518.aspx
    XMLNS                 string             `xml:"xmlns,attr"`
    ServiceName           string             `xml:"ServiceName"`
    Label                 string             `xml:"Label"`
    Description           string             `xml:"Description,omitempty"`
    Location              string             `xml:"Location"`
    AffinityGroup         string             `xml:"AffinityGroup,omitempty"`
    GeoReplicationEnabled string             `xml:"GeoReplicationEnabled,omitempty"`
    ExtendedProperties    []ExtendedProperty `xml:"ExtendedProperties>ExtendedProperty"`
}

func (c *CreateStorageServiceInput) Serialize() (string, error) {
    return toxml(c)
}

// NewCreateStorageServiceInputWithLocation creates a location-based
// CreateStorageServiceInput, with all required fields filled out.
func NewCreateStorageServiceInputWithLocation(name, label, location string) *CreateStorageServiceInput {
    return &CreateStorageServiceInput{
        XMLNS:       XMLNS,
        ServiceName: name,
        Label:       base64.StdEncoding.EncodeToString([]byte(label)),
        Location:    location,
    }
}

type MetadataItem struct {
    XMLName xml.Name
    Value   string `xml:",chardata"`
}

func (item *MetadataItem) Name() string {
    return item.XMLName.Local
}

type Metadata struct {
    Items []MetadataItem `xml:",any"`
}

type Blob struct {
    Name                  string   `xml:"Name"`
    Snapshot              string   `xml:"Snapshot"`
    URL                   string   `xml:"Url"`
    LastModified          string   `xml:"Properties>Last-Modified"`
    ETag                  string   `xml:"Properties>Etag"`
    ContentLength         string   `xml:"Properties>Content-Length"`
    ContentType           string   `xml:"Properties>Content-Type"`
    BlobSequenceNumber    string   `xml:"Properties>x-ms-blob-sequence-number"`
    BlobType              string   `xml:"Properties>BlobType"`
    LeaseStatus           string   `xml:"Properties>LeaseStatus"`
    LeaseState            string   `xml:"Properties>LeaseState"`
    LeaseDuration         string   `xml:"Properties>LeaseDuration"`
    CopyID                string   `xml:"Properties>CopyId"`
    CopyStatus            string   `xml:"Properties>CopyStatus"`
    CopySource            string   `xml:"Properties>CopySource"`
    CopyProgress          string   `xml:"Properties>CopyProgress"`
    CopyCompletionTime    string   `xml:"Properties>CopyCompletionTime"`
    CopyStatusDescription string   `xml:"Properties>CopyStatusDescription"`
    Metadata              Metadata `xml:"Metadata"`
}

type BlobEnumerationResults struct {
    // http://msdn.microsoft.com/en-us/library/windowsazure/dd135734.aspx
    XMLName       xml.Name `xml:"EnumerationResults"`
    ContainerName string   `xml:"ContainerName,attr"`
    Prefix        string   `xml:"Prefix"`
    Marker        string   `xml:"Marker"`
    MaxResults    string   `xml:"MaxResults"`
    Delimiter     string   `xml:"Delimiter"`
    Blobs         []Blob   `xml:"Blobs>Blob"`
    BlobPrefixes  []string `xml:"Blobs>BlobPrefix>Name"`
    NextMarker    string   `xml:"NextMarker"`
}

func (b *BlobEnumerationResults) Deserialize(data []byte) error {
    return xml.Unmarshal(data, b)
}

type StorageAccountKeys struct {
    // See http://msdn.microsoft.com/en-us/library/windowsazure/ee460785.aspx
    XMLName   xml.Name `xml:"StorageService"`
    URL       string   `xml:"Url"`
    Primary   string   `xml:"StorageServiceKeys>Primary"`
    Secondary string   `xml:"StorageServiceKeys>Secondary"`
}

func (s *StorageAccountKeys) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

type ContainerEnumerationResults struct {
    // See http://msdn.microsoft.com/en-us/library/windowsazure/dd179352.aspx
    XMLName    xml.Name    `xml:"EnumerationResults"`
    Prefix     string      `xml:"Prefix"`
    Marker     string      `xml:"Marker"`
    MaxResults string      `xml:"MaxResults"`
    Containers []Container `xml:"Containers>Container"`
    NextMarker string      `xml:"NextMarker"`
}

func (s *ContainerEnumerationResults) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

type Container struct {
    XMLName    xml.Name   `xml:"Container"`
    Name       string     `xml:"Name"`
    URL        string     `xml:"URL"`
    Properties Properties `xml:"Properties"`
    Metadata   Metadata   `xml:"Metadata"`
}

type Properties struct {
    LastModified  string `xml:"Last-Modified"`
    ETag          string `xml:"Etag"`
    LeaseStatus   string `xml:"LeaseStatus"`
    LeaseState    string `xml:"LeaseState"`
    LeaseDuration string `xml:"LeaseDuration"`
}

// An enumeration-lite type to define from which list (committed, uncommitted,
// latest) to get a block during a PutBlockList Storage API operation.
type BlockListType string

const (
    BlockListUncommitted BlockListType = "Uncommitted"
    BlockListCommitted   BlockListType = "Committed"
    BlockListLatest      BlockListType = "Latest"
)

// Payload for the PutBlockList operation.
type BlockList struct {
    XMLName xml.Name `xml:"BlockList"`
    Items   []*BlockListItem
}

func (s *BlockList) Serialize() ([]byte, error) {
    return marshalXML(s)
}

// Add a BlockListItem to a BlockList.
func (s *BlockList) Add(blockType BlockListType, blockID string) {
    base64ID := base64.StdEncoding.EncodeToString([]byte(blockID))
    item := NewBlockListItem(blockType, base64ID)
    s.Items = append(s.Items, item)
}

type BlockListItem struct {
    XMLName xml.Name
    BlockID string `xml:",chardata"`
}

// Create a new BlockListItem.
func NewBlockListItem(blockType BlockListType, blockID string) *BlockListItem {
    return &BlockListItem{
        XMLName: xml.Name{Local: string(blockType)},
        BlockID: blockID,
    }
}

func (item *BlockListItem) Type() BlockListType {
    switch BlockListType(item.XMLName.Local) {
    case BlockListUncommitted:
        return BlockListUncommitted
    case BlockListCommitted:
        return BlockListCommitted
    case BlockListLatest:
        return BlockListLatest
    }
    panic(fmt.Errorf("type not recognized: %s", item.XMLName.Local))
}

// GetBlockList result struct.
type Block struct {
    Name string `xml:"Name"`
    Size string `xml:"Size"`
}

type GetBlockList struct {
    XMLName           xml.Name `xml:"BlockList"`
    CommittedBlocks   []Block  `xml:"CommittedBlocks>Block"`
    UncommittedBlocks []Block  `xml:"UncommittedBlocks>Block"`
}

func (g *GetBlockList) Deserialize(data []byte) error {
    return xml.Unmarshal(data, g)
}

//
// Operation Services bits
//

const (
    InProgressOperationStatus = "InProgress"
    SucceededOperationStatus  = "Succeeded"
    FailedOperationStatus     = "Failed"
)

type Operation struct {
    ID             string `xml:"ID"`
    Status         string `xml:"Status"`
    HTTPStatusCode int    `xml:"HttpStatusCode"`
    ErrorCode      string `xml:"Error>Code"`
    ErrorMessage   string `xml:"Error>Message"`
}

func (o *Operation) Deserialize(data []byte) error {
    return xml.Unmarshal(data, o)
}
